;****************************************************************************             
;*** CAMCTRL.ASM
;*** VISCA CONTROL WITH JOYSTICK FOR PICDEV BOARD
;*** Author: (C)-2001 Christian Mohr (Christian@Mohr.se)
;****************************************************************************            
;*** CPU: 16C84
;*** CLOCK: 10MhZ Xtal
;****************************************************************************            


        Radix   DEC
        EXPAND

        include "dev_fam.inc"
        include "p16cxx.inc"


        __CONFIG 09h

;********************************************************************************************************
;				Setup RS-232 Parameters
;********************************************************************************************************
_ClkIn		equ	10000000 	; Input Clock Frequency is 10 Mhz
_BaudRate       set     9600            ; Baud Rate (bits per second) is 9600
_DataBits	set	8		; 8 bit data, can be 1 to 8
_StopBits	set	1		; 1 Stop Bit, 2 Stop Bits is not implemented

#define _PARITY_ENABLE  FALSE           ; NO Parity
#define _ODD_PARITY	FALSE		; EVEN Parity, if Parity enabled
#define _USE_RTSCTS     FALSE           ; NO Hardware Handshaking is Used
        include "rs232.h"


;****************************************************************************             
;*** PORT DECLARATIONS
;****************************************************************************            

#define I2C_D	PORTA,0		; I2C Data
#define I2C_C	PORTA,1		; I2C Clock
;#define LCD_R	PORTA,1		; LCD REGISTER SELECT
;#define LCD_E	PORTA,2		; LCD ENABLE
;#define TX	PORTA,3		; RS232 TX
;#define RX	PORTA,4		; RS232 RX

#define PB0	PORTB,0		; 
#define PB1	PORTB,1		; 
#define PB2	PORTB,2		; 
#define PB3	PORTB,3		; 
#define PB4	PORTB,4		; 
#define PB5	PORTB,5		; 
#define PB6	PORTB,6		; 
#define PB7	PORTB,7		; 

;****************************************************************************             
;*** RAM DECLARATIONS
;****************************************************************************            

; SERIAL
TxReg           set     0x19
RxReg           set     0x1A
RxTemp          set     0x1B
SerialStatus    set     0x1C
BitCount        set     0x1D
ExtraBitCount   set     0x1E
SaveWReg        set     0x1F
SaveStatus      set     0x20

; END SERIAL

S_P		set	0x22
S_T		set	0x23
P_T		set	0x24
MODE		set	0x25
D0      	set     0x26    ; A/D DATA 0
D1      	set     0x27    ; A/D DATA 1
C0      	set     0x28    ; Character 0
C1      	set     0x29    ; Character 1
_N      	set     0x2A    ; I/O BIT COUNTER
IOBYTE  	set     0x2B    ; I/O DATA STORAGE
T3		set     0x2C   ; TEMP. STORAGE
T2		set     0x2D   ; TEMP. STORAGE
T1		set     0x2E   ; TEMP. STORAGE
T0		set     0x2F   ; TEMP. STORAGE


;****************************************************************************             
;*** Vectors
;****************************************************************************            

        ORG     0x00
	goto	Start
        ORG     0x04
	goto    Interrupt


;****************************************************************************             
;*** 500 us delay
;****************************************************************************
; USES: T0, T1
D500us  MOVLW   2                  ;.4 us
        MOVWF   T0                 ;.4 us
D500us1 MOVLW   .80                ;.4 us
        MOVWF   T1                 ;.4 us
D500us2 DECFSZ  T1,1               ;.4 us * 206 * 6 = 494,4us
        GOTO    D500us2
        DECFSZ  T0,1               ;.4 us * 6
        GOTO    D500us1
        NOP                        ;.4
        NOP                        ;.4
        RETURN                     ;.8 us


;****************************************************************************             
;*** 60 us delay
;****************************************************************************
; USES: T0
D60us   MOVLW   .70                ;.4 us
        MOVWF   T0                 ;.4 us
D60us1  DECFSZ  T0,1               ;1.2 us * 48 = 57.6
        GOTO    D60us1		   ; 
	NOP			   ;.4 us
	NOP			   ;.4 us
        RETURN                     ;.8 us


;****************************************************************************
;*** "Normal" Delay  500ms
;****************************************************************************
; USES: (T0),(T1),T2,T3
; CALL: D500us
D500ms  MOVLW   .20
        MOVWF   T2
D500ms1 MOVLW   0x0A
        MOVWF   T3
D500ms2 DECFSZ  T3,1
        GOTO    D500ms3
        GOTO    D500ms4
        GOTO    D500ms3
D500ms3 CALL    D500us
        CALL    D500us
        GOTO    D500ms2
D500ms4 DECFSZ  T2,1
        GOTO    D500ms1
	RETURN

;****************************************************************************
;*** Delay  100ms
;****************************************************************************
; USES: (T0),(T1),T2,T3
; CALL: D500us
D100ms  MOVLW   .04
        MOVWF   T2
D100ms1 MOVLW   0x0A
        MOVWF   T3
D100ms2 DECFSZ  T3,1
        GOTO    D100ms3
        GOTO    D100ms4
        GOTO    D100ms3
D100ms3 CALL    D500us
        CALL    D500us
        GOTO    D100ms2
D100ms4 DECFSZ  T2,1
        GOTO    D100ms1
	RETURN



;*********************************************************************************************************
;***  I2C COMMUNICATION
;*********************************************************************************************************
; USES: _N, IOBYTE
; CALL: D60us, D500us


;*** Initialize I2C Bus
I2CINIT	BSF     STATUS,RP0     ; Select Page 1 for TrisB access
        BCF     I2C_C
        BSF     I2C_D
	BCF     STATUS,RP0
        BSF     I2C_C
        BSF     I2C_D
	RETURN


;*** OUTPUT 1 ON I2C BUS (PIN = INPUT)
I2C1:	BSF	STATUS, RP0        
        BSF     I2C_D		; high impedance
	BCF	STATUS, RP0                
	RETURN

;*** OUTPUT 0 ON I2CBUS
I2C0:   BCF     I2C_D
	BSF	STATUS, RP0        
        BCF     I2C_D		; low impedance zero
        BCF	STATUS, RP0
        BCF     I2C_D
	RETURN

;*** SEND I2C START CONDITION
I2CSTA:	CALL	I2C1
        CALL    D60us
        BSF     I2C_C
        CALL    D500us
        CALL    I2C0
        CALL    D60us
        BCF     I2C_C
	RETURN

;*** SEND I2C STOP CONDITION
I2CSTO: CALL    I2C0
        CALL    D60us
        BSF     I2C_C
        CALL    D60us
	CALL	I2C1
	RETURN

;*** WRITE BYTE TO I2C BUS
I2CWB:  MOVWF   IOBYTE
        MOVLW   .8
        MOVWF	_N
I2CWB1: RLF     IOBYTE, F
	BTFSS	STATUS, C
	GOTO	I2CWB3
	GOTO	I2CWB4
I2CWB2:	DECFSZ	_N, F
	GOTO	I2CWB1
	CALL	I2C1		; read ACK bit...
        CALL    D60us
        BSF     I2C_C
        CALL    D60us
        BCF     I2C_C
        CALL    D60us
	RETURN
I2CWB3:	CALL	I2C0		; bring DATA_PIN low
        CALL    D60us
        BSF     I2C_C
        CALL    D60us
        BCF     I2C_C
        GOTO    I2CWB2
I2CWB4:	CALL	I2C1
        CALL    D60us
        BSF     I2C_C
        CALL    D60us
        BCF     I2C_C
        GOTO    I2CWB2

;*** READ BYTE FROM I2C BUS
I2CRB:  CALL    I2C1
        MOVLW   .8
	MOVWF	 _N
        CLRF    IOBYTE
I2CRB1: BSF     I2C_C
        CALL    D60us
        BSF     STATUS,C
        BTFSS   I2C_D
        BCF     STATUS,C
        RLF     IOBYTE,F
        BCF     I2C_C
        CALL    D60us
        DECFSZ  _N, F
        GOTO    I2CRB1
        CALL    I2C0
        CALL    D60us
        BSF     I2C_C     ; ACK-bit
        CALL    D60us
        BCF     I2C_C
        CALL    D60us
        CALL    I2C1
        CALL    D60us
        MOVFW   IOBYTE          ; return the result in W
	RETURN

;*** READ LAST BYTE FROM I2C BUS (NO ACK)
I2CRL:  CALL    I2C1
        MOVLW   .8
	MOVWF	 _N
        CLRF    IOBYTE
I2CRL1: BSF     I2C_C
        CALL    D60us
        BSF     STATUS,C
        BTFSS   I2C_D
        BCF     STATUS,C
        RLF     IOBYTE,F
        BCF     I2C_C
        CALL    D60us
        DECFSZ  _N, F
        GOTO    I2CRL1
        CALL    I2C0
        CALL    D60us
        BSF     I2C_C     ; ACK-bit
        CALL    D60us
        CALL    I2C1
        MOVFW   IOBYTE          ; return the result in W
	RETURN


;*********************************************************************************************************
;**** BIN2HEX
;**** W = IN/OUT
;*********************************************************************************************************
BIN2HEX:
        addlw   .48
	movwf	T1
        movlw   .58
	subwf	T1,w
	btfss	STATUS,C
	goto	B2H_1		; G vidare om <10 [0..9]
        movlw   .7              ; >=10 ---> +55 ---> [A..F]
	addwf	T1,f
B2H_1:  movf    T1,w
	RETURN




;*********************************************************************************************************
;***  Main Program
;*********************************************************************************************************

Start:  bsf     STATUS,RP0     ; Select Page 1 for TrisB access
;	bcf	BL0
	bsf	PB0
	bsf	PB1
	bsf	PB2
	bsf	PB3
	bsf	PB4
	bsf	PB5
	bsf	PB6
	bsf	PB7
	bcf	OPTREG,7
        bcf     STATUS,RP0     
	MOVLW	0xFF
	MOVWF	PORTB

	; END OF DEBUG-FLASH

	CALL	I2CINIT

	CALL	InitSerialPort

main	; SEND VISCA RESET
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x06
	CALL	TXWR
	MOVLW	0x05
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
	
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms

	; SEND VISCA HOME
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x06
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR

	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms

	CLRF	MODE

FOBAR:	
	BCF	MODE,0
	BCF	MODE,1
	CLRF	P_T
	CLRF	S_P
	CLRF	S_T

	CALL    I2CSTA
        MOVLW   0x90    ; address
        CALL    I2CWB
        MOVLW   0x44    ; No Output, 4xSE-input, AutoInc, Ch 0
        CALL    I2CWB
        MOVLW   0x00    ; Output Byte
        CALL    I2CWB
        CALL    I2CSTO

        CALL    I2CSTA
        MOVLW   0x91
        CALL    I2CWB

	CALL	I2CRB	; START CONVERT(?)
	CALL	I2CRB
	MOVWF	D0
	CALL	I2CRL
	MOVWF	D1

	; PROCESS X - DATA

	MOVLW	.101
	SUBWF	D0,W
	BTFSC	STATUS,C
	GOTO	X_P	
X_M:	; x = 0-100

	MOVWF	D0
	MOVLW	0xFF
	XORWF	D0,F

	MOVLW	0xF0
	ANDWF	D0,F
	RRF	D0,F
	RRF	D0,F
	RRF	D0,F
	RRF	D0,F

	MOVLW	0x10
	IORWF	P_T,F

	MOVF	D0,W
	ADDLW	0x01
	MOVWF	S_P
	BCF	STATUS,C
	RLF	S_P,F
	
	GOTO	X_X

X_P:	MOVLW	.155
	SUBWF	D0,W
	BTFSS	STATUS,C
	GOTO	X_C

	RRF	D0,F
	RRF	D0,F
	RRF	D0,F
	RRF	D0,F
	MOVLW	0x07
	ANDWF	D0,F

	MOVLW	0x20
	IORWF	P_T,F

	MOVF	D0,W
;	ADDLW	0x02
	MOVWF	S_P
	BCF	STATUS,C
	RLF	S_P,F

	GOTO	X_X
	
X_C:	MOVLW	0x10	; CENTER
	MOVWF	S_P
	MOVLW	0x30
	IORWF	P_T,F
	BSF	MODE,0

X_X:	; PROCESS Y - DATA

	MOVLW	.101
	SUBWF	D1,W
	BTFSC	STATUS,C
	GOTO	Y_P	
Y_M:	; y = 0-100

	MOVWF	D1
	MOVLW	0xFF
	XORWF	D1,F

	MOVLW	0xF0
	ANDWF	D1,F
	RRF	D1,F
	RRF	D1,F
	RRF	D1,F
	RRF	D1,F

	MOVLW	0x01
	IORWF	P_T,F

	MOVF	D1,W
	ADDLW	0x01
	MOVWF	S_T
	BCF	STATUS,C
	RLF	S_T,F

	GOTO	Y_X

Y_P:	MOVLW	.155
	SUBWF	D1,W
	BTFSS	STATUS,C
	GOTO	Y_C

	RRF	D1,F
	RRF	D1,F
	RRF	D1,F
	RRF	D1,F
	MOVLW	0x07
	ANDWF	D1,F

	MOVLW	0x02
	IORWF	P_T,F

	MOVF	D1,W
;	ADDLW	0x02
	MOVWF	S_T
	BCF	STATUS,C
	RLF	S_T,F

	GOTO	Y_X
	
Y_C:	MOVLW	0x10	; CENTER
	MOVWF	S_T
	MOVLW	0x03
	IORWF	P_T,F
	BSF	MODE,1

Y_X:
	CALL	D100ms
	CALL	D100ms

	BTFSC	MODE,2
	GOTO	NOMOV

	; SEND VISCA PACKET
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x06
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR

	; PAN SPEED
	MOVF	S_P,W
	CALL	TXWR

	; TILT SPEED
	MOVF	S_T,W
	CALL	TXWR

	; PAN MOVE?
	MOVF	P_T,W
	MOVWF	T0
	RRF	T0,F
	RRF	T0,F
	RRF	T0,F
	RRF	T0,W
	ANDLW	0x0F
	CALL	TXWR

	; TILT MOVE?
	MOVF	P_T,W
	ANDLW	0x0F
	CALL	TXWR

	MOVLW	0xFF
	CALL	TXWR

NOMOV:	; 0=1 & 1=1 -> NOMOV=1
	BSF	MODE,2
	BTFSS	MODE,0
	BCF	MODE,2
	BTFSS	MODE,1
	BCF	MODE,2


DOZOOM:	CALL	D100ms
;	CALL	D100ms
;	CALL	D100ms

	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x07
	CALL	TXWR


Z_I:	BTFSC	PB6
	GOTO	Z_O
	; ZOOM IN
	MOVLW	0x2F
	CALL	TXWR
	GOTO	Z_D

Z_O:	BTFSC	PB5
	GOTO	NO_Z
	; ZOOM OUT
	MOVLW	0x3F
	CALL	TXWR
	GOTO	Z_D

NO_Z:	MOVLW	0x00
	CALL	TXWR

Z_D:	MOVLW	0xFF
	CALL	TXWR

	CALL	D100ms


	BCF	MODE,7
	
PRE1:	BTFSC	PB0
	GOTO	xPRE1
	BTFSS	PB4
	GOTO	sPRE1
rPRE1:	BSF	MODE,7
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x02
	CALL	TXWR
	MOVLW	0x00	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
	GOTO	xPRE1
sPRE1:	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x00	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
xPRE1:	


PRE2:	BTFSC	PB1
	GOTO	xPRE2
	BTFSS	PB4
	GOTO	sPRE2
rPRE2:	BSF	MODE,7
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x02
	CALL	TXWR
	MOVLW	0x01	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
	GOTO	xPRE2
sPRE2:	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x01	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
xPRE2:	

PRE3:	BTFSC	PB2
	GOTO	xPRE3
	BTFSS	PB4
	GOTO	sPRE3
rPRE3:	BSF	MODE,7
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x02
	CALL	TXWR
	MOVLW	0x02	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
	GOTO	xPRE3
sPRE3:	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x02	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
xPRE3:	

PRE4:	BTFSC	PB3
	GOTO	xPRE4
	BTFSS	PB4
	GOTO	sPRE4
rPRE4:	BSF	MODE,7
	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x02
	CALL	TXWR
	MOVLW	0x03	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
	GOTO	xPRE4
sPRE4:	MOVLW	0x81
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x04
	CALL	TXWR
	MOVLW	0x3F
	CALL	TXWR
	MOVLW	0x01
	CALL	TXWR
	MOVLW	0x03	; MEMORY #
	CALL	TXWR
	MOVLW	0xFF
	CALL	TXWR
xPRE4:	

	BTFSS	MODE,7
	GOTO	NODEL
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
	CALL	D500ms
NODEL:


	GOTO	FOBAR







;********************************************************************************************************
;				RS-232 Routines
;********************************************************************************************************
;				Interrupt Service Routine
;
; Only RTCC Inturrupt Is used. RTCC Inturrupt is used as timing for Serial Port Receive & Transmit
; Since RS-232 is implemented only as a Half Duplex System, The RTCC is shared by both Receive &
; Transmit Modules.
;	Transmission :
;			RTCC is setup for Internal Clock increments and interrupt is generated when
;			RTCC overflows. Prescaler is assigned, depending on The INPUT CLOCK & the
;			desired BAUD RATE.
;	Reception :
;			When put in receive mode, RTCC is setup for external clock mode (FALLING EDGE)
;			and preloaded with 0xFF. When a Falling Edge is detected on RTCC Pin, RTCC
;			rolls over and an Interrupt is generated (thus Start Bit Detect). Once the start
;			bit is detected, RTCC is changed to INTERNAL CLOCK mode and RTCC is preloaded
;			with a certain value for regular timing interrupts to Poll RTCC Pin (i.e RX pin).
;
;********************************************************************************************************

Interrupt:
        btfss   INTCON,T0IF
	retfie				; other interrupt, simply return & enable GIE
;
; Save Status On INT : WREG & STATUS Regs
;
	movwf	SaveWReg
        swapf   STATUS,w               ; affects no STATUS bits : Only way OUT to save STATUS Reg ?????
	movwf	SaveStatus
;
	btfsc	_txmtProgress
	goto	_TxmtNextBit		; Txmt Next Bit
	btfsc	_rcvProgress
;	goto	_RcvNextBit		; Receive Next Bit
;	goto	_SBitDetected		; Must be start Bit
;
RestoreIntStatus:
	swapf	SaveStatus,w
        movwf   STATUS                  ; restore STATUS Reg
	swapf	SaveWReg, F		; save WREG
	swapf	SaveWReg,w		; restore WREG
        bcf     INTCON,T0IF
	retfie
;
;********************************************************************************************************
;
;
;
; Configure TX Pin as output, make sure TX Pin Comes up in high state on Reset
; Configure, RX_Pin (RTCC pin) as Input, which is used to poll data on reception
;
;  Program Memory :	9 locations
;  Cycles	  :	10
;********************************************************************************************************

InitSerialPort:
	clrf	SerialStatus
;
        bcf     STATUS,RP0                             ; select Page 0 for Port Access
	bsf	TX				; make sure TX Pin is high on powerup, use RB Port Pullup
        bsf     STATUS,RP0                             ; Select Page 1 for TrisB access
	bcf	TX				; set TX Pin As Output Pin, by modifying TRIS
  if _USE_RTSCTS
	bcf	_RTS				; RTS is output signal, controlled by PIC16Cxx
	bsf	_CTS				; CTS is Input signal, controlled by the host
  endif
;	bsf	RX_Pin				; set RX Pin As Input for reception
        bcf     STATUS,RP0                             ; select Page 0 for Port Access
	return
;
;********************************************************************************************************

	include "txmtr.asm"                     ; The Transmit routines are in file "txmtr.asm"
;	include "rcvr.asm"                      ; The Receiver Routines are in File "rcvr.asm"

;********************************************************************************************************

        END
